#!/bin/bash

set -e

function exec {
        echo
        echo $*
        $*
}

export PATH="..:$PATH"

CFLAGS="-std=c99" # i use c66 here
CFLAGS="$CFLAGS -Oz -mdynamic-no-pic -fomit-frame-pointer" # basic size opt flags
CFLAGS="$CFLAGS -ffast-math -mfpmath=387 -fsingle-precision-constant" # smaller floating point code & data
CFLAGS="$CFLAGS -fno-optimize-register-move -mtune=i386 -floop-optimize2" # optimizes for compression
CXXFLAGS="$CFLAGS -fno-exceptions -fno-rtti" # not used here but are a must for c++

# Compile
exec gcc $CFLAGS -c example.c -o example.o

# Default linker
exec gcc example.o -framework Carbon -framework CoreAudio -framework OpenGL -o example.ld

# Muncho
exec muncho -R2 -s 1320 example.o --no-args -fCarbon -o example.muncho


LSIZE=`wc -c < example.ld`
MSIZE=`wc -c < example.muncho`

PERCENT=`bc<<EOF
scale = 2
$MSIZE * 100 / $LSIZE
EOF
`

cat <<EOF

Sizes:
      ld: $LSIZE
  muncho: $MSIZE ($PERCENT%)
EOF
