#include <stdlib.h>
#include <ApplicationServices/ApplicationServices.h>
#include <OpenGL/OpenGL.h>
#include <OpenGL/glu.h>
#define deprecated // we don't want the deprecated warning about AudioDeviceAddIOProc
#include <CoreAudio/CoreAudio.h>
#undef deprecated
#include <Carbon/Carbon.h>


static int keyTest();
static OSStatus audioCallback(AudioDeviceID inDevice, const AudioTimeStamp *inNow, const AudioBufferList *inInputData, const AudioTimeStamp *inInputTime, AudioBufferList *outOutputData, const AudioTimeStamp *inOutputTime, void *appGlobals);

static CGLPixelFormatAttribute attribs[] = {
    kCGLPFADoubleBuffer,
    kCGLPFAFullScreen,
    kCGLPFADisplayMask, kCGNullDirectDisplay,
    0
};

static unsigned int audioChannels;
static unsigned int samplePosition;

int main()
{
    // Video setup
    CGCaptureAllDisplays();
    CGDisplayHideCursor(kCGNullDirectDisplay);
    
    CGLPixelFormatObj pixelFormatObj;
    GLint numPixelFormats;
    attribs[3] = CGDisplayIDToOpenGLDisplayMask(CGMainDisplayID());
    CGLChoosePixelFormat( attribs, &pixelFormatObj, &numPixelFormats );
    
    CGLContextObj openglContext;
    CGLCreateContext( pixelFormatObj, NULL, &openglContext );
	CGLSetCurrentContext( openglContext );
	CGLSetFullScreen( openglContext );
	
	// Enable vsync
	GLint swapInterval = 1;
	CGLSetParameter(openglContext, kCGLCPSwapInterval, &swapInterval); 
	
	// Audio setup
    unsigned long count;
    AudioDeviceID device;
    
    AudioStreamBasicDescription	deviceFormat;
    count = sizeof(device);
	AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &count, &device);
	count = sizeof(deviceFormat);
	AudioDeviceGetProperty(device, 0, false, kAudioDevicePropertyStreamFormat, &count, &deviceFormat);
	deviceFormat.mSampleRate = 44100;
	audioChannels = deviceFormat.mChannelsPerFrame;
	AudioDeviceSetProperty(device, 0,  0, false, kAudioDevicePropertyStreamFormat, count, &deviceFormat);
	AudioDeviceAddIOProc(device, audioCallback, NULL);
	AudioDeviceStart(device, audioCallback);
	
	// Main loop
	while(true)
	{
	    if(keyTest())
            exit(0);
            
        float time = samplePosition/44100.0;
        
        // coder colors
        glClearColor(cosf(time)+0.5,cosf((time + M_PI * 2/3))+0.5,cosf((time + M_PI * 4/3 ))+0.5,0);
        glClear(GL_COLOR_BUFFER_BIT);
        
	    CGLFlushDrawable(openglContext);
	}
}

int keyTest()
{
	KeyMap pressedKeys;
	GetKeys(pressedKeys);
    return ((unsigned int*)pressedKeys)[1] & 0x00200000; // test for escape
}

OSStatus audioCallback(AudioDeviceID inDevice, const AudioTimeStamp *inNow, const AudioBufferList *inInputData, const AudioTimeStamp *inInputTime, AudioBufferList *outOutputData, const AudioTimeStamp *inOutputTime, void *appGlobals)
{
    float *fillBuffer = (float*)outOutputData->mBuffers[0].mData;
	unsigned int size = outOutputData->mBuffers[0].mDataByteSize / audioChannels / sizeof(float);
    
    for(unsigned int i = 0; i < size; i++)
    {
        // coder "music"
        float sample = cosf(samplePosition/44100.0 * 440 * 2 * M_PI);
        
        for(unsigned int c = 0; c < audioChannels; c++)
            *(fillBuffer++) = sample;
        samplePosition++;
        
    }
    return 0;
}
